@@ -68,7 +68,7 @@ guile_corba_generic_make_type_name (const gchar *StudlyCaps)
                                   "gtype-name->class-name"));
     }
     
-    ret = scm_call_1 (de_studly_capsify, scm_makfrom0str (StudlyCaps));
+    ret = scm_call_1 (de_studly_capsify, scm_from_locale_string (StudlyCaps));
 
     return scm_to_locale_string (scm_symbol_to_string (ret));
 }
