@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset: 4 -*- */
 /* guile-gnome
- * Copyright (C) 2001 Martin Baulig <martin@gnome.org>
+ * Copyright (C) 2001, 2009 Martin Baulig <martin@gnome.org>
  * Copyright (C) 2003,2004,2008 Andy Wingo <wingo at pobox dot com>
  *
  * gparameter.c: Support for GParamSpec
@@ -67,6 +67,7 @@ SCM_SYMBOL (sym_boxed_type, "boxed-type");
 SCM_SYMBOL (sym_enum_type, "enum-type");
 SCM_SYMBOL (sym_flags_type, "flags-type");
 SCM_SYMBOL (sym_element_spec, "element-spec");
+SCM_SYMBOL (sym_is_a_type, "is-a-type");
 
 
 
@@ -218,6 +219,12 @@ static void scm_c_gparam_initialize_scm (SCM param, gpointer ppspec)
         SET (default_value, scm_from_uint (p->default_value));
     }
     
+    else if (G_IS_PARAM_SPEC_GTYPE (pspec)) {
+        GParamSpecGType *p = (GParamSpecGType *) pspec;
+        SET (is_a_type, ((p->is_a_type == G_TYPE_NONE)
+                         ? SCM_BOOL_F : scm_c_gtype_to_class (p->is_a_type)));
+    }
+
     else {
         g_warning ("param type not implemented: %s",
                    g_type_name (G_TYPE_FROM_INSTANCE (pspec)));
@@ -392,6 +399,12 @@ scm_c_gparam_construct (SCM instance, SCM initargs)
                                       scm_to_uint (REF (default_value)),
                                       flags);
     }
+    else if (param_type == G_TYPE_PARAM_GTYPE) {
+        SCM type = REF (is_a_type);
+        if (scm_is_true (type))
+            type = scm_c_gtype_class_to_gtype (type);
+        pspec = g_param_spec_gtype (name, nick, blurb, type, flags);
+    }
     else {
         scm_c_gruntime_error ("%gparam-construct",
                               "Can't create instance of ~A from initargs: ~A",
@@ -427,12 +440,17 @@ SCM_DEFINE (scm_sys_hacky_struct_ref, "%hacky-struct-ref",
   data = SCM_STRUCT_DATA (handle);
   p = scm_to_size_t (pos);
 
+#if SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION < 9
   layout_len = scm_i_symbol_length (layout);
   if (SCM_STRUCT_VTABLE_FLAGS (handle) & SCM_STRUCTF_LIGHT)
     /* no extra words */
     n_fields = layout_len / 2;
   else
     n_fields = data[scm_struct_i_n_words];
+#else
+  layout_len = scm_c_symbol_length (layout);
+  n_fields = layout_len / 2;
+#endif
   
   SCM_ASSERT_RANGE(1, pos, p < n_fields);
 
@@ -456,12 +474,17 @@ SCM_DEFINE (scm_sys_hacky_struct_set_x, "%hacky-struct-set!",
   data = SCM_STRUCT_DATA (handle);
   p = scm_to_size_t (pos);
 
+#if SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION < 9
   layout_len = scm_i_symbol_length (layout);
   if (SCM_STRUCT_VTABLE_FLAGS (handle) & SCM_STRUCTF_LIGHT)
     /* no extra words */
     n_fields = layout_len / 2;
   else
     n_fields = data[scm_struct_i_n_words];
+#else
+  layout_len = scm_c_symbol_length (layout);
+  n_fields = layout_len / 2;
+#endif
   
   SCM_ASSERT_RANGE(1, pos, p < n_fields);
 
