@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-basic-offset: 4 -*- */
 /* guile-gnome
- * Copyright (C) 2001 Martin Baulig <martin@gnome.org>
+ * Copyright (C) 2001, 2009, 2010 Martin Baulig <martin@gnome.org>
  * Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>
  *
  * gvalue.c: Support for GValue-based types
@@ -73,6 +73,8 @@ static GHashTable *gtype_instance_wrappers = NULL;
  * SCM representation of GValue*
  **********************************************************************/
 
+#if SCM_MAJOR_VERSION == 1 && SCM_MINOR_VERSION < 9
+#define scm_vtable_index_instance_finalize scm_struct_i_free
 static size_t
 scm_gvalue_struct_free (scm_t_bits * vtable, scm_t_bits * data)
 {
@@ -86,13 +88,26 @@ scm_gvalue_struct_free (scm_t_bits * vtable, scm_t_bits * data)
     scm_struct_free_light (vtable, data);
     return 0;
 }
+#else
+static void
+scm_gvalue_struct_free (SCM object)
+{
+    GValue *value = (GValue *) SCM_STRUCT_DATA_REF (object, 0);
+
+    if (value) {
+        DEBUG_ALLOC ("freeing GValue %p", value);
+        g_value_unset (value);
+        scm_gc_free (value, sizeof (GValue), "%gvalue");
+    }
+}
+#endif
 
 SCM_DEFINE (scm_sys_bless_gvalue_class, "%bless-gvalue-class", 1, 0, 0,
             (SCM class), "")
 {
     scm_t_bits *slots = SCM_STRUCT_DATA (class);
     scm_class_gvalue = scm_permanent_object (class);
-    slots[scm_struct_i_free] = (scm_t_bits)scm_gvalue_struct_free;
+    slots[scm_vtable_index_instance_finalize] = (scm_t_bits)scm_gvalue_struct_free;
     return SCM_UNSPECIFIED;
 }
 
@@ -222,10 +237,10 @@ scm_c_gvalue_ref (const GValue *gvalue)
 	return scm_from_ulong_long (g_value_get_uint64 (gvalue));
 
     case G_TYPE_FLOAT:
-	return scm_make_real ((double) g_value_get_float (gvalue));
+	return scm_from_double ((double) g_value_get_float (gvalue));
 
     case G_TYPE_DOUBLE:
-	return scm_make_real (g_value_get_double (gvalue));
+	return scm_from_double (g_value_get_double (gvalue));
 
     case G_TYPE_STRING:
         {
@@ -283,23 +298,15 @@ scm_c_gvalue_set (GValue *gvalue, SCM value)
     case G_TYPE_CHAR:
         if (SCM_CHARP (value))
             g_value_set_char (gvalue, SCM_CHAR (value));
-        else if (SCM_INUMP (value)
-                 && SCM_INUM (value) >= 0 && SCM_INUM (value) <= 127)
-            g_value_set_char (gvalue, SCM_INUM (value));
         else
-            scm_c_gruntime_error (FUNC_NAME, "Bad char value: ~A",
-                                  SCM_LIST1 (value));
+            g_value_set_char (gvalue, scm_to_int8 (value));
 	break;
 
     case G_TYPE_UCHAR:
         if (SCM_CHARP (value))
             g_value_set_uchar (gvalue, SCM_CHAR (value));
-        else if (SCM_INUMP (value)
-                 && SCM_INUM (value) >= 0 && SCM_INUM (value) <= 255)
-            g_value_set_uchar (gvalue, SCM_INUM (value));
         else
-            scm_c_gruntime_error (FUNC_NAME, "Bad uchar value: ~A",
-                                  SCM_LIST1 (value));
+            g_value_set_uchar (gvalue, scm_to_uint8 (value));
 	break;
 
     case G_TYPE_BOOLEAN:
@@ -308,38 +315,38 @@ scm_c_gvalue_set (GValue *gvalue, SCM value)
 	break;
 
     case G_TYPE_INT:
-	g_value_set_int (gvalue, SCM_NUM2INT (2, value));
+	g_value_set_int (gvalue, scm_to_int (value));
 	break;
 
     case G_TYPE_UINT:
-	g_value_set_uint (gvalue, SCM_NUM2UINT (2, value));
+	g_value_set_uint (gvalue, scm_to_uint (value));
 	break;
 
     case G_TYPE_LONG:
-	g_value_set_long (gvalue, SCM_NUM2LONG (2, value));
+	g_value_set_long (gvalue, scm_to_long (value));
 	break;
 
     case G_TYPE_ULONG:
-	g_value_set_ulong (gvalue, SCM_NUM2ULONG (2, value));
+	g_value_set_ulong (gvalue, scm_to_ulong (value));
 	break;
 
     case G_TYPE_INT64:
-	g_value_set_int64 (gvalue, SCM_NUM2INT (2, value));
+	g_value_set_int64 (gvalue, scm_to_int64 (value));
 	break;
 
     case G_TYPE_UINT64:
-	g_value_set_uint64 (gvalue, SCM_NUM2UINT (2, value));
+	g_value_set_uint64 (gvalue, scm_to_uint64 (value));
 	break;
 
     case G_TYPE_FLOAT: {
-	double x = scm_num2dbl (value, FUNC_NAME);
+	double x = scm_to_double (value);
 	SCM_ASSERT_RANGE (2, value, (- G_MAXFLOAT < x) && (x < G_MAXFLOAT));
 	g_value_set_float (gvalue, (float) x);
 	break;
     }
 
     case G_TYPE_DOUBLE:
-	g_value_set_double (gvalue, scm_num2dbl (value, FUNC_NAME));
+	g_value_set_double (gvalue, scm_to_double (value));
 	break;
 
     case G_TYPE_STRING:
