@@ -1,5 +1,5 @@
 ;; guile-gnome
-;; Copyright (C) 2003,2004,2008 Andy Wingo <wingo at pobox dot com>
+;; Copyright (C) 2003,2004,2008,2009,2011 Andy Wingo <wingo at pobox dot com>
 ;; Copyright (C) 2004,2007 Andreas Rottmann <rotty at debian dot org>
 
 ;; This program is free software; you can redistribute it and/or    
@@ -895,8 +895,8 @@ invoked manually in a wrapset as needed."
    (wrap-func type) ", " (unwrap-func type) ");\n"))
 
 (define-macro (make-custom-wrapper type wrap-form)
-  `(let ((ctype (,c-type-name ,type))
-         (wrap-func (,wrap-func ,type)))
+  `(let ((ctype ((@@ (gnome gw support gobject) c-type-name) ,type))
+         (wrap-func ((@@ (gnome gw support gobject) wrap-func) ,type)))
      (lambda (scm-var c-var)
        (list
         "static SCM " wrap-func " (const GValue* gvalue) {\n"
@@ -907,8 +907,8 @@ invoked manually in a wrapset as needed."
         "}\n"))))
      
 (define-macro (make-custom-unwrapper type unwrap-form)
-  `(let ((ctype (,c-type-name ,type))
-         (unwrap-func (,unwrap-func ,type)))
+  `(let ((ctype ((@@ (gnome gw support gobject) c-type-name) ,type))
+         (unwrap-func ((@@ (gnome gw support gobject) unwrap-func) ,type)))
      (lambda (scm-var c-var)
        (list
         "static void " unwrap-func " (SCM " scm-var ", GValue* gvalue) {\n"
@@ -970,15 +970,18 @@ SCM and C values, respectively. For example:
                                  (GStudlyCapsExpand ctype)))
          (wrap-func (string-append "gw__gvalue_" func-infix "_wrap"))
          (unwrap-func (string-append "gw__gvalue_" func-infix "_unwrap")))
-    `(let ((t (make ,<gobject-custom-boxed-type>
+    `(let ((t (make (@@ (gnome gw support gobject)
+                        <gobject-custom-boxed-type>)
                 #:ctype ,ctype
                 #:gtype-id ,gtype
                 #:c-type-name ,pname
                 #:wrapped "Custom"
                 #:wrap-func ,wrap-func
                 #:unwrap-func ,unwrap-func)))
-       (slot-set! t 'wrap (,make-custom-wrapper t ,wrap))
-       (slot-set! t 'unwrap (,make-custom-unwrapper t ,unwrap))
+       (slot-set! t 'wrap ((@@ (gnome gw support gobject) make-custom-wrapper)
+                           t ,wrap))
+       (slot-set! t 'unwrap ((@@ (gnome gw support gobject) make-custom-unwrapper)
+                             t ,unwrap))
        (add-type! ws t)
        (add-type-alias! ws ,pname (name t)))))
 
@@ -1038,7 +1041,7 @@ example:
                       \"scm_from_gst_fraction\"
                       \"scm_to_gst_fraction\")
 @end lisp"
-  `(let ((t (make ,<gobject-custom-gvalue-type>
+  `(let ((t (make (@@ (gnome gw support gobject) <gobject-custom-gvalue-type>)
               #:ctype ,ctype
               #:gtype-id ,gtype
               #:c-type-name "GValue*"
@@ -1105,18 +1108,20 @@ example:
               #:use-module (gnome gw support modules)
               ,@(if (slot-ref wrapset 'shlib-abs?)
                     '(#:use-module (g-wrap config))
-                    '()))
+                    '())
+              ,@(append-map (lambda (mod) `(#:use-module ,mod))
+                            (filter-map module (wrapsets-depended-on wrapset))))
 
            (let ((wrapset-name-c-sym (any-str->c-sym-str
                                       (symbol->string (name wrapset)))))
-             `(dynamic-call ,(++ "gw_init_wrapset_" wrapset-name-c-sym)
-                            (dynamic-link
-                             ,(if (slot-ref wrapset 'shlib-abs?)
-                                  `(string-append
-                                    *g-wrap-shlib-dir*
-                                    ,(slot-ref wrapset 'shlib-path))
-                                  (slot-ref wrapset 'shlib-path)))))
-           
+             `(eval-when (eval load compile)
+                (load-extension ,(if (slot-ref wrapset 'shlib-abs?)
+                                     `(string-append
+                                       *g-wrap-shlib-dir*
+                                       ,(slot-ref wrapset 'shlib-path))
+                                     (slot-ref wrapset 'shlib-path))
+                                ,(++ "gw_init_wrapset_" wrapset-name-c-sym))))
+
            ;; This is what we avoid:
            ;;   `(export ',@(module-exports wrapset))
            ;; Instead we do this lovely hack:
