@@ -1,5 +1,5 @@
 /* guile-gnome
- * Copyright (C) 2003 Andy Wingo <wingo at pobox dot com>
+ * Copyright (C) 2003, 2010 Andy Wingo <wingo at pobox dot com>
  *
  * gtk-support.c: Customizations for guile-gtk
  *
@@ -24,16 +24,9 @@
  */
 
 #include <string.h>
+#include "guile-support.h"
 #include "gtk-support.h"
 
-static char*
-scm_to_locale_string_dynwind (SCM s)
-{
-    char *ret = scm_to_locale_string (s);
-    scm_dynwind_free (ret);
-    return ret;
-}
-    
 
 /* python-gtk defines a nice series of classes to access rows as elements of a
    sequence and to access their fields via a sequence interface. I wish we could
@@ -131,6 +124,8 @@ _wrap_gtk_action_group_add_actions (GtkActionGroup *action_group,
                                     (scm_c_resolve_module ("gnome gobject"),
                                      "gtype-instance-signal-connect"));
 
+    scm_dynwind_begin (0);
+
     for (; SCM_NNULLP (entries); entries = SCM_CDR (entries)) {
         entry = SCM_CAR (entries);
         SCM_VALIDATE_NONEMPTYLIST (1, entry);
@@ -151,11 +146,13 @@ _wrap_gtk_action_group_add_actions (GtkActionGroup *action_group,
         if (5 < l && SCM_NFALSEP ((x = scm_list_ref (entry, scm_from_uint16 (5)))))
             scm_call_3 (connect,
                         scm_c_gtype_instance_to_scm ((GTypeInstance*)action),
-                        scm_str2symbol ("activate"), x);
+                        scm_from_locale_symbol ("activate"), x);
 
         gtk_action_group_add_action_with_accel (action_group, action, accelerator);
         g_object_unref (action);
     }
+
+    scm_dynwind_end ();
 }
 #undef FUNC_NAME
 
@@ -178,6 +175,8 @@ _wrap_gtk_action_group_add_toggle_actions (GtkActionGroup *action_group,
                                     (scm_c_resolve_module ("gnome gobject"),
                                      "gtype-instance-signal-connect"));
 
+    scm_dynwind_begin (0);
+
     for (; SCM_NNULLP (entries); entries = SCM_CDR (entries)) {
         entry = SCM_CAR (entries);
         SCM_VALIDATE_NONEMPTYLIST (1, entry);
@@ -198,7 +197,7 @@ _wrap_gtk_action_group_add_toggle_actions (GtkActionGroup *action_group,
         if (5 < l && SCM_NFALSEP ((x = scm_list_ref (entry, scm_from_uint16 (5)))))
             scm_call_3 (connect,
                         scm_c_gtype_instance_to_scm ((GTypeInstance*)action),
-                        scm_str2symbol ("activate"), x);
+                        scm_from_locale_symbol ("activate"), x);
 
         GET_ACTION_BOOL (is_active, 6);
         gtk_toggle_action_set_active (action, is_active);
@@ -206,6 +205,8 @@ _wrap_gtk_action_group_add_toggle_actions (GtkActionGroup *action_group,
         gtk_action_group_add_action_with_accel (action_group, GTK_ACTION (action), accelerator);
         g_object_unref (action);
     }
+
+    scm_dynwind_end ();
 }
 #undef FUNC_NAME
 
@@ -241,6 +242,8 @@ _wrap_gtk_action_group_add_radio_actions (GtkActionGroup *action_group,
     len = scm_ilength (entries);
     raes = g_new0 (GtkRadioActionEntry, len);
 
+    scm_dynwind_begin (0);
+
     for (i=0; SCM_NNULLP (entries); entries = SCM_CDR (entries), i++) {
         entry = SCM_CAR (entries);
         SCM_VALIDATE_NONEMPTYLIST (1, entry);
@@ -260,6 +263,8 @@ _wrap_gtk_action_group_add_radio_actions (GtkActionGroup *action_group,
                                         G_CALLBACK (action_group_radio_actions_callback),
                                         SCM_TO_GPOINTER (on_change));
 
+    scm_dynwind_end ();
+
     g_free (raes);
 }
 #undef FUNC_NAME
@@ -377,41 +382,41 @@ _wrap_gtk_list_store_remove (GtkListStore *store, GtkTreeIter *iter)
 GtkTreeIter*
 _wrap_gtk_list_store_insert (GtkListStore *store, gint position)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_list_store_insert (store, new, position);
-    return new;
+    GtkTreeIter new;
+    gtk_list_store_insert (store, &new, position);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_list_store_insert_before (GtkListStore *store, GtkTreeIter *sibling)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_list_store_insert_before (store, new, sibling);
-    return new;
+    GtkTreeIter new;
+    gtk_list_store_insert_before (store, &new, sibling);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_list_store_insert_after (GtkListStore *store, GtkTreeIter *sibling)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_list_store_insert_after (store, new, sibling);
-    return new;
+    GtkTreeIter new;
+    gtk_list_store_insert_after (store, &new, sibling);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_list_store_prepend (GtkListStore *store)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_list_store_prepend (store, new);
-    return new;
+    GtkTreeIter new;
+    gtk_list_store_prepend (store, &new);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_list_store_append (GtkListStore *store)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_list_store_append (store, new);
-    return new;
+    GtkTreeIter new;
+    gtk_list_store_append (store, &new);
+    return gtk_tree_iter_copy (&new);
 }
 
 static void
@@ -508,11 +513,11 @@ _wrap_gtk_stock_lookup (const gchar *stock_id)
     GtkStockItem item;
 	
     if (gtk_stock_lookup (stock_id, &item)) {
-        return SCM_LIST5 (scm_makfrom0str (item.stock_id),
-                          scm_makfrom0str (item.label),
+        return SCM_LIST5 (scm_from_locale_string (item.stock_id),
+                          scm_from_locale_string (item.label),
                           scm_from_uint (item.modifier),
                           scm_from_uint (item.keyval),
-                          scm_makfrom0str (item.translation_domain));
+                          scm_from_locale_string (item.translation_domain));
     }
 
     return SCM_BOOL_F;
@@ -524,9 +529,13 @@ void
 _wrap_gtk_text_buffer_set_text (GtkTextBuffer *buf, SCM stext)
 #define FUNC_NAME "gtk-text-buffer-set-text"
 {
+    char *utf8_text;
+    size_t utf8_len;
+
     SCM_VALIDATE_STRING (2, stext);
     scm_dynwind_begin (0);
-    gtk_text_buffer_set_text (buf, scm_to_locale_string_dynwind (stext), scm_c_string_length (stext));
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    gtk_text_buffer_set_text (buf, utf8_text, utf8_len);
     scm_dynwind_end ();
 }
 #undef FUNC_NAME
@@ -535,9 +544,13 @@ void
 _wrap_gtk_text_buffer_insert (GtkTextBuffer *buf, GtkTextIter* iter, SCM stext)
 #define FUNC_NAME "gtk-text-buffer-insert"
 {
+    char *utf8_text;
+    size_t utf8_len;
+
     SCM_VALIDATE_STRING (3, stext);
     scm_dynwind_begin (0);
-    gtk_text_buffer_insert (buf, iter, scm_to_locale_string_dynwind (stext), scm_c_string_length (stext));
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    gtk_text_buffer_insert (buf, iter, utf8_text, utf8_len);
     scm_dynwind_end ();
 }
 #undef FUNC_NAME
@@ -546,9 +559,13 @@ void
 _wrap_gtk_text_buffer_insert_at_cursor (GtkTextBuffer *buf, SCM stext)
 #define FUNC_NAME "gtk-text-buffer-insert-at-cursor"
 {
+    char *utf8_text;
+    size_t utf8_len;
+
     SCM_VALIDATE_STRING (2, stext);
     scm_dynwind_begin (0);
-    gtk_text_buffer_insert_at_cursor (buf, scm_to_locale_string_dynwind (stext), scm_c_string_length (stext));
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    gtk_text_buffer_insert_at_cursor (buf, utf8_text, utf8_len);
     scm_dynwind_end ();
 }
 #undef FUNC_NAME
@@ -558,12 +575,15 @@ _wrap_gtk_text_buffer_insert_interactive (GtkTextBuffer *buf, GtkTextIter* iter,
                                                    SCM stext, gboolean default_editable)
 #define FUNC_NAME "gtk-text-buffer-insert-interactive"
 {
+    char *utf8_text;
+    size_t utf8_len;
     gboolean ret;
 
     SCM_VALIDATE_STRING (3, stext);
     scm_dynwind_begin (0);
-    ret = gtk_text_buffer_insert_interactive (buf, iter, scm_to_locale_string_dynwind (stext),
-                                              scm_c_string_length (stext), default_editable);
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    ret = gtk_text_buffer_insert_interactive (buf, iter, utf8_text, utf8_len,
+					      default_editable);
     scm_dynwind_end ();
     return ret;
 }
@@ -574,11 +594,14 @@ _wrap_gtk_text_buffer_insert_interactive_at_cursor (GtkTextBuffer *buf, SCM stex
                                                     gboolean default_editable)
 #define FUNC_NAME "gtk-text-buffer-insert-interactive-at-cursor"
 {
+    char *utf8_text;
+    size_t utf8_len;
     gboolean ret;
+
     SCM_VALIDATE_STRING (2, stext);
     scm_dynwind_begin (0);
-    ret = gtk_text_buffer_insert_interactive_at_cursor (buf, scm_to_locale_string_dynwind (stext),
-                                                        scm_c_string_length (stext),
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    ret = gtk_text_buffer_insert_interactive_at_cursor (buf, utf8_text, utf8_len,
                                                         default_editable);
     scm_dynwind_end ();
     return ret;
@@ -590,6 +613,8 @@ _wrap_gtk_text_buffer_insert_with_tags (GtkTextBuffer *buf, GtkTextIter* iter,
                                         SCM stext, GList* tag_list)
 #define FUNC_NAME "gtk-text-buffer-insert-with-tags"
 {
+    char *utf8_text;
+    size_t utf8_len;
     GtkTextIter start;
     GList *walk;
     gint start_offset;
@@ -599,8 +624,8 @@ _wrap_gtk_text_buffer_insert_with_tags (GtkTextBuffer *buf, GtkTextIter* iter,
     SCM_VALIDATE_STRING (3, stext);
     scm_dynwind_begin (0);
     start_offset = gtk_text_iter_get_offset (iter);
-    gtk_text_buffer_insert (buf, iter, scm_to_locale_string_dynwind (stext),
-                            scm_c_string_length (stext));
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    gtk_text_buffer_insert (buf, iter, utf8_text, utf8_len);
     gtk_text_buffer_get_iter_at_offset (buf, &start, start_offset);
     
     for (walk = tag_list; walk; walk = walk->next)
@@ -615,6 +640,8 @@ _wrap_gtk_text_buffer_insert_with_tags_by_name (GtkTextBuffer *buf, GtkTextIter*
                                                 SCM stext, GList* tag_list)
 #define FUNC_NAME "gtk-text-buffer-insert-with-tags-by-name"
 {
+    char *utf8_text;
+    size_t utf8_len;
     GtkTextIter start;
     GList *walk;
     gint start_offset;
@@ -624,8 +651,8 @@ _wrap_gtk_text_buffer_insert_with_tags_by_name (GtkTextBuffer *buf, GtkTextIter*
     SCM_VALIDATE_STRING (3, stext);
     scm_dynwind_begin (0);
     start_offset = gtk_text_iter_get_offset (iter);
-    gtk_text_buffer_insert (buf, iter, scm_to_locale_string_dynwind (stext),
-                            scm_c_string_length (stext));
+    utf8_text = scm_to_utf8_stringn_dynwind (stext, &utf8_len);
+    gtk_text_buffer_insert (buf, iter, utf8_text, utf8_len);
     gtk_text_buffer_get_iter_at_offset (buf, &start, start_offset);
     
     for (walk = tag_list; walk; walk = walk->next)
@@ -642,120 +669,112 @@ GtkTextIter*
 _wrap_gtk_text_buffer_get_iter_at_line_offset (GtkTextBuffer *buf, gint line_number,
                                                gint char_offset)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_iter_at_line_offset (buf, iter, line_number, char_offset);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_line_offset (buf, &iter, line_number, char_offset);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_iter_at_line_index (GtkTextBuffer *buf, gint line_number,
                                               gint byte_index)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_iter_at_line_index (buf, iter, line_number, byte_index);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_line_index (buf, &iter, line_number, byte_index);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_iter_at_offset (GtkTextBuffer *buf, gint char_offset)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_iter_at_offset (buf, iter, char_offset);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_offset (buf, &iter, char_offset);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_iter_at_line (GtkTextBuffer *buf, gint line_number)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_iter_at_line (buf, iter, line_number);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_line (buf, &iter, line_number);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_start_iter (GtkTextBuffer *buf)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_start_iter (buf, iter);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_start_iter (buf, &iter);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_end_iter (GtkTextBuffer *buf)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_end_iter (buf, iter);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_end_iter (buf, &iter);
+    return gtk_text_iter_copy (&iter);
 }
 
 SCM
 _wrap_gtk_text_buffer_get_bounds (GtkTextBuffer *buf)
 {
-    GtkTextIter *start, *end;
+    GtkTextIter start, end;
     SCM sstart, send;
     
-    start = g_new0 (GtkTextIter, 1);
-    end = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_bounds (buf, start, end);
-    sstart = scm_c_gvalue_new_take_boxed (GTK_TYPE_TEXT_ITER, start);
-    send = scm_c_gvalue_new_take_boxed (GTK_TYPE_TEXT_ITER, end);
+    gtk_text_buffer_get_bounds (buf, &start, &end);
+    sstart = scm_c_gvalue_new_from_boxed (GTK_TYPE_TEXT_ITER, &start);
+    send = scm_c_gvalue_new_from_boxed (GTK_TYPE_TEXT_ITER, &end);
     return scm_values (SCM_LIST2 (sstart, send));
 }
 
 SCM
 _wrap_gtk_text_buffer_get_selection_bounds (GtkTextBuffer *buf)
 {
-    GtkTextIter *start, *end;
+    GtkTextIter start, end;
     SCM sstart, send;
     
-    start = g_new0 (GtkTextIter, 1);
-    end = g_new0 (GtkTextIter, 1);
-    if (gtk_text_buffer_get_selection_bounds (buf, start, end)) {
-        sstart = scm_c_gvalue_new_take_boxed (GTK_TYPE_TEXT_ITER, start);
-        send = scm_c_gvalue_new_take_boxed (GTK_TYPE_TEXT_ITER, end);
+    if (gtk_text_buffer_get_selection_bounds (buf, &start, &end)) {
+        sstart = scm_c_gvalue_new_from_boxed (GTK_TYPE_TEXT_ITER, &start);
+        send = scm_c_gvalue_new_from_boxed (GTK_TYPE_TEXT_ITER, &end);
         return scm_values (SCM_LIST2 (sstart, send));
     }
-    g_free (start);
-    g_free (end);
     return scm_values (SCM_LIST2 (SCM_BOOL_F, SCM_BOOL_F));
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_iter_at_mark (GtkTextBuffer *buf, GtkTextMark* mark)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_iter_at_mark (buf, iter, mark);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_mark (buf, &iter, mark);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTextIter*
 _wrap_gtk_text_buffer_get_iter_at_child_anchor (GtkTextBuffer *buf,
                                                 GtkTextChildAnchor* anchor)
 {
-    GtkTextIter* iter = g_new0 (GtkTextIter, 1);
-    gtk_text_buffer_get_iter_at_child_anchor (buf, iter, anchor);
-    return iter;
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_child_anchor (buf, &iter, anchor);
+    return gtk_text_iter_copy (&iter);
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_model_get_iter (GtkTreeModel *model, GtkTreePath *path)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
+    GtkTreeIter new;
     
-    if (gtk_tree_model_get_iter (model, new, path))
-        return new;
-    g_free (new);
+    if (gtk_tree_model_get_iter (model, &new, path))
+        return gtk_tree_iter_copy (&new);
     return NULL;
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_model_get_iter_first (GtkTreeModel *model) 
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
+    GtkTreeIter new;
     
-    if (gtk_tree_model_get_iter_first (model, new))
-        return new;
-    g_free (new);
+    if (gtk_tree_model_get_iter_first (model, &new))
+        return gtk_tree_iter_copy (&new);
     return NULL;
 }
 
@@ -784,59 +803,51 @@ GList*
 _wrap_gtk_tree_model_iter_children (GtkTreeModel *model, GtkTreeIter *iter)
 {
     GList *list = NULL;
-    GtkTreeIter *prev = g_new0 (GtkTreeIter, 1);
+    GtkTreeIter prev;
     
-    if (gtk_tree_model_iter_children (model, prev, iter)) {
+    if (gtk_tree_model_iter_children (model, &prev, iter)) {
         while (1) {
-            GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
+            list = g_list_prepend (list, gtk_tree_iter_copy (&prev));
 
-            list = g_list_prepend (list, prev);
-            *new = *prev;
-
-            if (!gtk_tree_model_iter_next (model, new)) {
-                g_free (new);
+            if (!gtk_tree_model_iter_next (model, &prev)) {
                 return g_list_reverse (list);
             }
-            prev = new;
         }
     }
-    g_free (prev);
     return NULL;
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_model_iter_parent (GtkTreeModel *model, GtkTreeIter *iter)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
+    GtkTreeIter new;
     
-    if (gtk_tree_model_iter_parent (model, new, iter))
-        return new;
-    g_free (new);
+    if (gtk_tree_model_iter_parent (model, &new, iter))
+        return gtk_tree_iter_copy (&new);
     return NULL;
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_model_iter_nth_child (GtkTreeModel *model, GtkTreeIter *iter, gint n)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
+    GtkTreeIter new;
     
-    if (gtk_tree_model_iter_nth_child (model, new, iter, n))
-        return new;
-    g_free (new);
+    if (gtk_tree_model_iter_nth_child (model, &new, iter, n))
+        return gtk_tree_iter_copy (&new);
     return NULL;
 }
 
 SCM
 _wrap_gtk_tree_selection_get_selected (GtkTreeSelection *selection)
 {
-    GtkTreeIter *iter = g_new0 (GtkTreeIter, 1);
+    GtkTreeIter iter;
     GtkTreeModel *model = NULL;
     
-    if (gtk_tree_selection_get_selected (selection, &model, iter)) {
+    if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
         SCM smodel, siter;
         g_object_ref (model);
         smodel = scm_c_gtype_instance_to_scm ((GTypeInstance*)model);
-        siter = scm_c_gvalue_new_take_boxed (GTK_TYPE_TREE_ITER, iter);
+        siter = scm_c_gvalue_new_from_boxed (GTK_TYPE_TREE_ITER, &iter);
         return scm_values (SCM_LIST2 (smodel, siter));
     }
     return scm_values (SCM_LIST2 (SCM_BOOL_F, SCM_BOOL_F));
@@ -894,48 +905,72 @@ _wrap_gtk_tree_store_remove (GtkTreeStore *store, GtkTreeIter *iter)
 GtkTreeIter*
 _wrap_gtk_tree_store_insert (GtkTreeStore *store, GtkTreeIter *parent, gint position)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_tree_store_insert (store, new, parent, position);
-    return new;
+    GtkTreeIter new;
+    gtk_tree_store_insert (store, &new, parent, position);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_store_insert_before (GtkTreeStore *store, GtkTreeIter *parent,
                                     GtkTreeIter *sibling)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_tree_store_insert_before (store, new, parent, sibling);
-    return new;
+    GtkTreeIter new;
+    gtk_tree_store_insert_before (store, &new, parent, sibling);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_store_insert_after (GtkTreeStore *store, GtkTreeIter *parent,
                                    GtkTreeIter *sibling)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_tree_store_insert_after (store, new, parent, sibling);
-    return new;
+    GtkTreeIter new;
+    gtk_tree_store_insert_after (store, &new, parent, sibling);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_store_prepend (GtkTreeStore *store, GtkTreeIter *parent)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_tree_store_prepend (store, new, parent);
-    return new;
+    GtkTreeIter new;
+    gtk_tree_store_prepend (store, &new, parent);
+    return gtk_tree_iter_copy (&new);
 }
 
 GtkTreeIter*
 _wrap_gtk_tree_store_append (GtkTreeStore *store, GtkTreeIter *parent)
 {
-    GtkTreeIter *new = g_new0 (GtkTreeIter, 1);
-    gtk_tree_store_append (store, new, parent);
-    return new;
+    GtkTreeIter new;
+    gtk_tree_store_append (store, &new, parent);
+    return gtk_tree_iter_copy (&new);
+}
+
+SCM
+_wrap_gtk_tree_view_get_path_at_pos (GtkTreeView *treeview,
+                                     gint x, gint y)
+{
+    GtkTreePath *path;
+    GtkTreeViewColumn *column;
+    gint cell_x, cell_y;
+
+    if (gtk_tree_view_get_path_at_pos (treeview, x, y, &path, &column,
+                                       &cell_x, &cell_y)) {
+        SCM spath = guile_gtk_tree_path_to_scm (path);
+        SCM scolumn = scm_c_gtype_instance_to_scm (column);
+
+        gtk_tree_path_free (path);
+
+        return scm_values (SCM_LIST4 (spath, scolumn, scm_from_int (cell_x),
+                                      scm_from_int (cell_y)));
+    }
+
+    return scm_values (
+        SCM_LIST4 (SCM_BOOL_F, SCM_BOOL_F, SCM_BOOL_F, SCM_BOOL_F));
 }
 
 static void
-cell_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
-                GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
+with_cell_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
+                     GtkTreeModel *tree_model, GtkTreeIter *iter,
+                     gpointer data)
 {
     SCM proc, scolumn, scell, smodel, siter;
     proc = GPOINTER_TO_SCM (data);
@@ -947,6 +982,15 @@ cell_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
     scm_call_4 (proc, scolumn, scell, smodel, siter);
 }
 
+static void
+cell_data_func (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
+                GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data)
+{
+    scm_dynwind_guile_v__p_p_p_p_p (scm_with_guile, with_cell_data_func,
+                                    tree_column, cell, tree_model, iter,
+                                    data);
+}
+
 void
 _wrap_gtk_tree_view_column_set_cell_data_func (GtkTreeViewColumn *tree_column,
                                                GtkCellRenderer *cell_renderer,
@@ -1012,10 +1056,14 @@ gtk_widget_get_window (GtkWidget *widget)
 }
 
 GdkRectangle*
-gtk_widget_get_allocation (GtkWidget *widget)
+_wrap_gtk_widget_get_allocation (GtkWidget *widget)
 {
     GdkRectangle *ret = g_new (GdkRectangle, 1);
-    *ret = widget->allocation;
+#if GTK_CHECK_VERSION(2,18,0)
+    gtk_widget_get_allocation (widget, (GtkAllocation*)ret);
+#else
+     *ret = widget->allocation;
+#endif
     return ret;
 }
 
